package aicare.net.cn.sdk.tmpsrepositoryandroid;

import android.Manifest;
import android.bluetooth.BluetoothAdapter;
import android.content.Intent;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;
import android.text.TextUtils;
import android.view.View;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.TextView;

import com.pingwang.tpmslibrary.TpmsScan;

import java.util.ArrayList;
import java.util.EnumMap;
import java.util.List;
import java.util.Set;

import aicare.net.cn.sdk.tmpsrepositoryandroid.utils.Config;
import aicare.net.cn.sdk.tmpsrepositoryandroid.utils.Configs;
import aicare.net.cn.sdk.tmpsrepositoryandroid.utils.L;
import aicare.net.cn.sdk.tmpsrepositoryandroid.utils.ParseData;
import aicare.net.cn.sdk.tmpsrepositoryandroid.utils.SPUtils;
import aicare.net.cn.sdk.tmpsrepositoryandroid.views.SetIdDialog;
import androidx.appcompat.app.AppCompatActivity;

public class MainActivity extends AppCompatActivity implements View.OnClickListener, TpmsScan.TpmsScanListener, OnIdSetListener {

    private Button btn_get_info, btn_clear_log, btn_stop_scan;
    private TextView tv_left_front, tv_right_front, tv_left_rear, tv_right_rear;
    private ListView lv_show_log;
    private LinearLayout ll_set_id;
    private String leftFront, rightFront, leftRear, rightRear;

    private List<String> infoList;
    private ArrayAdapter listAdapter;
    private TpmsScan mTpmsScan;
    private EnumMap<Config.DevicePosition, String> deviceIdMap;
    private String[] mDeviceIdS;

    /**
     * 需要申请的权限
     */
    private String[] LOCATION_PERMISSION = new String[]{Manifest.permission.ACCESS_FINE_LOCATION};

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        L.isDebug = true;
        initData();
        initViews();
        initEvents();
    }

    @Override
    protected void onResume() {
        super.onResume();
        if (mTpmsScan != null) {
            if (!BluetoothAdapter.getDefaultAdapter().isEnabled()) {
                showBLEDialog();
            }
        }
    }


    private void initData() {
        infoList = new ArrayList<>();
        deviceIdMap = new EnumMap<>(Config.DevicePosition.class);
        setDeviceIdMap();

        mTpmsScan = new TpmsScan(MainActivity.this);
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
            requestPermissions(LOCATION_PERMISSION, 1);
        }

    }

    private void setDeviceIdMap() {
        leftFront = (String) SPUtils.get(this, Configs.LEFT_FRONT, "");
        leftRear = (String) SPUtils.get(this, Configs.LEFT_REAR, "");
        rightFront = (String) SPUtils.get(this, Configs.RIGHT_FRONT, "");
        rightRear = (String) SPUtils.get(this, Configs.RIGHT_REAR, "");
        deviceIdMap.put(Config.DevicePosition.LEFT_FRONT, leftFront);
        deviceIdMap.put(Config.DevicePosition.LEFT_REAR, leftRear);
        deviceIdMap.put(Config.DevicePosition.RIGHT_FRONT, rightFront);
        deviceIdMap.put(Config.DevicePosition.RIGHT_REAR, rightRear);
    }

    private void initViews() {
        btn_stop_scan = (Button) findViewById(R.id.btn_stop_scan);
        btn_get_info = (Button) findViewById(R.id.btn_get_info);
        btn_clear_log = (Button) findViewById(R.id.btn_clear_log);

        tv_left_front = (TextView) findViewById(R.id.tv_left_front);
        tv_right_front = (TextView) findViewById(R.id.tv_right_front);
        tv_left_rear = (TextView) findViewById(R.id.tv_left_rear);
        tv_right_rear = (TextView) findViewById(R.id.tv_right_rear);
        setTyreId();

        ll_set_id = (LinearLayout) findViewById(R.id.ll_set_id);

        lv_show_log = (ListView) findViewById(R.id.lv_show_log);
        listAdapter = new ArrayAdapter<>(this, android.R.layout.simple_list_item_1, infoList);
        lv_show_log.setAdapter(listAdapter);
    }

    private void setTyreId() {
        if (TextUtils.isEmpty(leftFront)) {
            tv_left_front.setText(R.string.please_set_id);
        } else {
            tv_left_front.setText(getString(R.string.left_front_hint) + ":" + leftFront);
        }
        if (TextUtils.isEmpty(rightFront)) {
            tv_right_front.setText(R.string.please_set_id);
        } else {
            tv_right_front.setText(getString(R.string.left_rear_hint) + ":" + rightFront);
        }
        if (TextUtils.isEmpty(leftRear)) {
            tv_left_rear.setText(R.string.please_set_id);
        } else {
            tv_left_rear.setText(getString(R.string.right_front_hint) + ":" + leftRear);
        }
        if (TextUtils.isEmpty(rightRear)) {
            tv_right_rear.setText(R.string.please_set_id);
        } else {
            tv_right_rear.setText(getString(R.string.right_rear_hint) + ":" + rightRear);
        }
    }

    private void initEvents() {
        btn_stop_scan.setOnClickListener(this);
        btn_get_info.setOnClickListener(this);
        btn_clear_log.setOnClickListener(this);
        ll_set_id.setOnClickListener(this);
    }


    private static final int REQUEST_ENABLE_BT = 2;

    private void showBLEDialog() {
        final Intent enableIntent = new Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE);
        startActivityForResult(enableIntent, REQUEST_ENABLE_BT);
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        if (mTpmsScan != null) {
            mTpmsScan.stopScan();
        }
    }


    @Override
    public void onClick(View v) {
        switch (v.getId()) {
            case R.id.btn_stop_scan:
                if (mTpmsScan != null) {
                    mTpmsScan.stopScan();
                }
                break;
            case R.id.btn_get_info:
                if (deviceIdMap.isEmpty() || noId()) {
                    mTpmsScan.startScan();
                } else {
                    if (mTpmsScan != null && deviceIdMap != null) {
                        mDeviceIdS = null;
                        if (getUuidStrings()) {
                            mTpmsScan.startScan(mDeviceIdS);
                        }
                    }
                }
                break;

            case R.id.btn_clear_log:
                infoList.clear();
                listAdapter.notifyDataSetChanged();
                break;
            case R.id.ll_set_id:
                new SetIdDialog(this, deviceIdMap, this).show();
                break;
        }
    }


    @Override
    public void onGetData(byte[] bytes, String mac, String deviceId, int rssi, float pressure, int pressureUnit, float battery, int temp, int tempUnit, int status, float mcuVersion, int year,
                          int month, int day, float bleVersion) {
        StringBuffer buffer = new StringBuffer();
        for (Config.DevicePosition devicePosition : deviceIdMap.keySet()) {
            if (TextUtils.equals(deviceId, deviceIdMap.get(devicePosition))) {
                buffer.append("Location：");
                buffer.append(getPosition(devicePosition));
                buffer.append(", ");
            }
        }

        Config.DeviceState deviceState = Config.DeviceState.NORMAL;
        switch (status) {
            case 1:
                deviceState = Config.DeviceState.LEAK;
                break;
            case 2:
                deviceState = Config.DeviceState.INFLATE;
                break;
            case 3:
                deviceState = Config.DeviceState.START;
                break;
            case 4:
                deviceState = Config.DeviceState.POWER_ON;
                break;
            case 5:
                deviceState = Config.DeviceState.WEAK_UP;
                break;
        }
        buffer.append("deviceId：");
        buffer.append(deviceId);
        buffer.append(", battery：");
        buffer.append(battery);
        buffer.append(", pressure：");
        buffer.append(pressure);
        buffer.append(", Temp：");
        buffer.append(temp);
        buffer.append(", status：");
        buffer.append(getState(deviceState));
        buffer.append(", time：");
        buffer.append(ParseData.getTime());
        sendMsg(buffer.toString());

    }


    private String getPosition(Config.DevicePosition devicePosition) {
        switch (devicePosition) {
            case LEFT_FRONT:
                return getString(R.string.left_front_hint);
            case RIGHT_FRONT:
                return getString(R.string.left_rear_hint);
            case LEFT_REAR:
                return getString(R.string.right_front_hint);
            case RIGHT_REAR:
                return getString(R.string.right_rear_hint);
            default:
                return "";
        }
    }


    private String getState(Config.DeviceState deviceState) {
        switch (deviceState) {
            case NORMAL:
                return getString(R.string.normal);
            case LEAK:
                return getString(R.string.leak);
            case INFLATE:
                return getString(R.string.inflatable);
            case START:
                return getString(R.string.start);
            case POWER_ON:
                return getString(R.string.power_on);
            case WEAK_UP:
                return getString(R.string.wake);
            case TEMP_ERROR:
                return getString(R.string.temp_err);
            case BATTERY_ERROR:
                return getString(R.string.battery_err);
            case UNKNOWN:
                return getString(R.string.unknown);
            default:
                return "";
        }
    }

    @Override
    public void onIdSet() {
        setDeviceIdMap();
        if (noId()) {
            if (mTpmsScan != null) {
                mTpmsScan.stopScan();
            }
        }
        setTyreId();
    }

    @Override
    public void cancel() {
        if (mTpmsScan != null) {
            mTpmsScan.stopScan();
        }
    }

    /**
     * 没有设置ID重新扫描
     */
    private boolean noId() {
        if (deviceIdMap != null && !deviceIdMap.isEmpty()) {
            if (TextUtils.isEmpty(deviceIdMap.get(Config.DevicePosition.LEFT_FRONT)) && TextUtils.isEmpty(deviceIdMap.get(Config.DevicePosition.LEFT_REAR)) && TextUtils
                    .isEmpty(deviceIdMap.get(Config.DevicePosition.RIGHT_FRONT)) && TextUtils.isEmpty(deviceIdMap.get(Config.DevicePosition.RIGHT_REAR))) {
                return true;
            }
        }
        return false;
    }

    private void showInfo(String str) {
        StringBuilder stringBuilder = new StringBuilder();
        stringBuilder.append(infoList.size() + 1);
        stringBuilder.append(".\n");
        stringBuilder.append(str);
        infoList.add(stringBuilder.toString());
        listAdapter.notifyDataSetChanged();
        lv_show_log.setSelection(infoList.size() - 1);
    }

    private Handler handler = new android.os.Handler(Looper.getMainLooper()) {
        @Override
        public void handleMessage(Message msg) {
            String result = msg.getData().getString(RESULT);
            if (!TextUtils.isEmpty(result)) {
                showInfo(result);
            }
        }
    };

    private final static String RESULT = "RESULT";

    private void sendMsg(String str) {
        Message msg = new Message();
        Bundle bundle = new Bundle();
        bundle.putString(RESULT, str);
        msg.setData(bundle);
        handler.sendMessage(msg);
    }


    private boolean getUuidStrings() {
        if (deviceIdMap != null) {
            Set<Config.DevicePosition> devicePositions = deviceIdMap.keySet();
            mDeviceIdS = new String[devicePositions.size()];
            int i = 0;
            for (Config.DevicePosition devicePosition : devicePositions) {
                mDeviceIdS[i] = deviceIdMap.get(devicePosition);
                i++;
            }
            return true;
        }
        return false;
    }

}
